#!build/env/bin/python
# Copyright (c) 2013 Cloudera, Inc. All rights reserved.
#
# Basic SysSQL modules

import psutil
import datetime
import time
import logging
import socket
import platform
import netifaces
try:
  import json
except:
  import simplejson as json

from cmf.syssql.table import SimpleModule, is_valid_type

__VERSION__ = '0.1'

LOG = logging.getLogger(__name__)

hostname = socket.getfqdn()

class MountsModule(SimpleModule):
  """Mount points"""
  name = "mounts"

  columns = [
    "hostname",
    "device",
    "mountpoint",
    "fstype",
    "total",
    "used",
    "free"
  ]

  def generate(self):
    for p in psutil.disk_partitions():
      yield p, psutil.disk_usage(p.mountpoint)

  def access(self, datum, column):
    mount, usage = datum
    attr = self.columns[column]

    if attr == "hostname":
      return hostname

    try:
      a = mount.__getattribute__(attr)
    except AttributeError:
      a = usage.__getattribute__(attr)
    return a

class HostsModule(SimpleModule):
  """Host metadata"""
  name = "hosts"

  columns = [
    "hostname",
    "os",
    "os_version",
    "uptime",
    "num_cpus"
  ]

  def generate(self):
    data = dict(
      hostname=hostname,
      os=platform.system(),
      os_version=platform.version(),
      uptime=float(file("/proc/uptime").read().split(" ")[0]),
      num_cpus=psutil.NUM_CPUS
    )

    return iter([ data ])

  def access(self, datum, column):
    return datum.get(self.columns[column])

class ThreadsModule(SimpleModule):
  """Threads"""
  name = "threads"

  columns = [
    "hostname",
    "pid",
    "tid",
    "cpu_user",
    "cpu_system"
  ]

  def generate(self):
    for process in psutil.get_process_list():
      # Threads can finish, so we try/except
      # everything.
      try:
        for thread in process.get_threads():
          try:
            yield hostname, process.pid, thread.id, thread.user_time, thread.system_time
          except:
            pass
      except:
        pass

class ProcessesModule(SimpleModule):
  """Processes"""
  name = "processes"

  columns = [
    "hostname",
    "pid",
    "name",
    "cmdline",
    "create_time",
    "exe",
    "cwd",
    "gid",
    "ppid",
    "uid",
    "username",
    "cpu_user",
    "cpu_system",
    "mem_rss",
    "mem_vms"
  ]

  def generate(self):
    return iter(psutil.get_process_list())

  def access(self, process, column):
    attr = self.columns[column]

    if attr == "hostname":
      return hostname

    try:
      if attr == "cpu_user":
        data = process.get_cpu_times().user
      elif attr == "cpu_system":
        data = process.get_cpu_times().system
      elif attr == "mem_rss":
        data = process.get_memory_info().rss
      elif attr == "mem_vms":
        data = process.get_memory_info().vms
      else:
        data = process.__getattribute__(attr)
    except:
      # Could get "access denied" from psutil.
      return None

    # Convert invalid types
    if data is not None and attr == "cmdline":
      data = " ".join(data)
    if not is_valid_type(data):
      data = str(data)
    if attr == "create_time":
      data = datetime.datetime.fromtimestamp(data).isoformat()

    return data

class ConnectionsModule(SimpleModule):
  name = "connections"
  columns = ["hostname", "pid", "fd", "family", "type", "local_host", "local_port", "remote_host", "remote_port", "status"]

  def generate(self):
    for p in psutil.get_process_list():
      try:
        for c in p.get_connections():
          yield (p.pid, c)
      except:
        # LOG exceptions?
        pass

  def access(self, datum, column):
    attr = self.columns[column]

    pid, conn = datum

    if attr == "local_host":
      return conn.local_address[0]
    elif attr == "local_port":
      return conn.local_address[1]
    elif attr == "remote_host":
      return conn.remote_address[0]
    elif attr == "remote_port":
      return conn.remote_address[1]
    elif attr == "pid":
      return pid
    elif attr == "hostname":
      return hostname
    else:
      return conn.__getattribute__(attr)

class ZeroThroughNineModule(SimpleModule):
  """Test module to check that iteration works correctly."""
  name = "zero_through_nine"
  columns = ["hostname", "i"]

  def generate(self):
    LOG.info("Zero-through-nine test log entry.")
    for i in xrange(10):
      yield hostname, i

class OneThroughTenThousandModule(SimpleModule):
  """Ten thousand integers, for join scale testing."""
  name = "one_through_ten_thousand"
  columns = ["hostname", "i"]

  def generate(self):
    for i in xrange(10000):
      yield hostname, i + 1

class InterfacesModule(SimpleModule):
  """
  Network Interfaces.
  """
  # Additional notes:
  """
  >>> netifaces.interfaces()
  ['lo', 'eth0', 'wlan0', 'vmnet1', 'vmnet8', 'tun0']
  >>> netifaces.ifaddresses('wlan0')
  {17: [{'broadcast': 'ff:ff:ff:ff:ff:ff', 'addr': '24:77:03:47:63:98'}], 2: [{'broadcast': '192.168.11.255', 'netmask': '255.255.255.0', 'addr': '192.168.11.34'}], 10: [{'netmask': 'ffff:ffff:ffff:ffff::', 'addr': 'fe80::2677:3ff:fe47:6398%wlan0'}]}
  address_families {0: 'AF_UNSPEC', 1: 'AF_FILE', 2: 'AF_INET', 3: 'AF_AX25', 4: 'AF_IPX', 5: 'AF_APPLETALK', 6: 'AF_NETROM', 7: 'AF_BRIDGE', 8: 'AF_ATMPVC', 9: 'AF_X25', 10: 'AF_INET6', 11: 'AF_ROSE', 12: 'AF_DECnet', 13: 'AF_NETBEUI', 14: 'AF_SECURITY', 15: 'AF_KEY', 16: 'AF_NETLINK', 17: 'AF_PACKET', 18: 'AF_ASH', 19: 'AF_ECONET', 20: 'AF_ATMSVC', 22: 'AF_SNA', 23: 'AF_IRDA', 24: 'AF_PPPOX', 25: 'AF_WANPIPE', 31: 'AF_BLUETOOTH', 34: 'AF_ISDN'}
  """

  name = "network_interfaces"
  columns = [ "hostname", "iface", "hw_addr", "inet_addr", "inet_bcast", "inet_mask",
        "inet6_addr", "inet6_bcast", "inet6_mask" ]

  def generate(self):
    for iface in netifaces.interfaces():
      addr = netifaces.ifaddresses(iface)
      inet = addr.get(netifaces.AF_INET, [dict()])[0]
      inet6 = addr.get(netifaces.AF_INET6, [dict()])[0]
      yield (
        hostname,
        iface,
        addr.get(netifaces.AF_NETLINK, dict()).get("addr", ""),
        inet.get('addr'),
        inet.get('broadcast'),
        inet.get('netmask'),
        inet6.get('addr'),
        inet6.get('bcast'),
        inet6.get('mask'))
